#!/usr/bin/env bash
#SBATCH --account=ACCOUNT
#SBATCH --time=00:30:00
#SBATCH --partition=dc-gpu-devel
#SBATCH --gres=gpu:4
#SBATCH --nodes=1
#SBATCH --ntasks-per-node=4
#SBATCH --cpus-per-task=32
#SBATCH --output=tmp/log/out.%j
#SBATCH --error=tmp/log/err.%j

echo "Use working directory"
pwd

echo "Enabling environment..."
source env/activate.sh

export CUDA_VISIBLE_DEVICES=0,1,2,3
# export NCCL_DEBUG=INFO

# Compute node execution
srun python cli/train_example.py \
    --name=Ragib \
    --train_size=1024 \
    --val_size=64 \
    --batch_size=8 \
    --crop_size=182 \
    --patch_size=128 \
    --num_workers=4 \
    --learning_rate=0.001 \
    --gpus=0 \
    --num_nodes=1 \
    --sync_batchnorm=true \
    --max_epochs=10 \
    --accelerator=ddp \
    --fast_dev_run=false \
    --log_every_n_steps=8 \

# Local execution
# mpiexec -n 1 python cli/train_example.py \
	
