import torch.nn as nn
from torch.nn import functional as F
import torch
import pytorch_lightning as pl

import segmentation_models_pytorch as smp


class TestModule(pl.LightningModule):

    @staticmethod
    def add_model_specific_args(parent_parser):
        parser = parent_parser.add_argument_group("TestModule")
        parser.add_argument('--learning_rate', type=float, default=1e-3)
        parser.add_argument('--name', type=str, default='model')
        return parent_parser

    @classmethod
    def from_argparse_args(cls, args):
        dict_args = vars(args)
        return cls(**dict_args)

    def __init__(self, learning_rate=1e-3, name='model', depth=3, **kwargs):
        super().__init__()
        self.save_hyperparameters()
        self.learning_rate = learning_rate
        self.name = name

        # Define the model
        self.model = smp.Unet(
            encoder_name="resnet34",  # Also consider using smaller or larger encoders
            encoder_weights="imagenet",  # Do the pretrained weights help? Try with or without
            in_channels=1,  # We use 1 chanel transmittance as input
            classes=1,  # classes == output channels. We use one output channel for cyto data
        )
        self.loss_f = torch.nn.L1Loss()

    def forward(self, x):
        x = self.model(x)
        return x

    def training_step(self, batch, batch_idx):
        # Structure of the batch: {'pli_image': pli_image, 'cyto_image': stained_image}
        cyto_imag_generated = self.forward(batch['pli_image'])
        loss = self.loss_f(cyto_imag_generated, batch['cyto_image'])
        self.log('train_loss', loss)
        return loss

    def validation_step(self, batch, batch_idx):
        # Do it the same as in training
        cyto_imag_generated = self.forward(batch['pli_image'])
        loss = self.loss_f(cyto_imag_generated, batch['cyto_image'])
        self.log("val_loss", loss)

    def configure_optimizers(self):
        optimizer = torch.optim.Adam(self.parameters(), lr=self.learning_rate)
        return optimizer
