### Package Import ###

'''
#Argeparse
import argparse
parser = argparse.ArgumentParser()
parser.add_argument('--foo', help='foo help')
args = parser.parse_args()

print(args)
'''


import sys, os
from argparse import ArgumentParser

import numpy as np
import pandas as pd
import h5py as h5

import torch
import pytorch_lightning as pl
from pytorch_lightning.loggers import TensorBoardLogger
from pytorch_lightning.callbacks import ModelCheckpoint

import pli
import pli.image as im

# Distributed
from atlasmpi import MPI

comm = MPI.COMM_WORLD

# Add code path
sys.path.insert(0, "code/")

import utils
import model
import data


def main():
    parser = ArgumentParser()
    parser = pl.Trainer.add_argparse_args(parser)
    parser = data.TestDataModule.add_argparse_args(parser)
    parser = model.TestModule.add_model_specific_args(parser)
    parser.add_argument('--log_dir', type=str, default='doc/tensorboard/')
    parser.add_argument('--ckpt_dir', type=str, default='tmp/ckpt/')
    parser.add_argument('--save_every_n_epochs', type=int, default=None)
    args = parser.parse_args()
    dict_args = vars(args)

    print("Create model")
    test_model = model.TestModule.from_argparse_args(args)
    print(f"Model '{test_model.name}' loaded")

    print("Load train data and create Data Module")
    dm = data.TestDataModule.from_argparse_args(args)
    dm.prepare_data()

    print("Define logger")
    logger = TensorBoardLogger(
        save_dir=args.log_dir,
        name=test_model.name,
        version=None,
    )

    print("Define checkpoints")
    checkpoint_callback = ModelCheckpoint(
        dirpath=os.path.join(args.ckpt_dir, args.name, f'version_{logger.version}'),
        filename=f"{test_model.name}_{logger.version}_" + '{epoch}_{val_loss:.3g}_{train_loss:.3g}',
        every_n_epochs=args.save_every_n_epochs,
        monitor='val_loss',
        verbose=True,
        save_last=True,
    )

    print("Create trainer")
    trainer = pl.Trainer.from_argparse_args(
        args,
        checkpoint_callback=True,
        callbacks=[checkpoint_callback],
        logger=logger,
    )

    print(f"Start training on {args.gpus} GPUs")
    trainer.logger.log_hyperparams(dict_args)
    trainer.fit(test_model, datamodule=dm)

    print("Done")
    exit(0)


if __name__ == '__main__':
    if comm.rank == 0:
        print("Initialize training...")

        # Environment information
        print("PLI-Helpers", pli.__version__)
        print("Pytorch-Lightning", pl.__version__)
        print("Pytorch", torch.__version__)
        print("GPUs", torch.cuda.device_count())

        print("")
    main()
